import os

from atsphinx.audioplayer import __version__ as version
from atsphinx.mini18n import get_template_dir as get_mini18n_template_dir

# -- Project information
project = "atsphinx-audioplayer"
copyright = "2024, Kazuya Takei"
author = "Kazuya Takei"
release = version

# -- General configuration
extensions = [
    "atsphinx.audioplayer",
    "atsphinx.mini18n",
    "sphinx.ext.intersphinx",
    "sphinx.ext.todo",
    "sphinx_copybutton",
    "sphinx_design",
]
templates_path = ["_templates", get_mini18n_template_dir()]
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# -- Options for i18n
locale_dirs = ["_locales"]
gettext_compact = False
gettext_language_team = "Kazuya Takei <myself@attakei.net>"
gettext_last_translator = os.environ.get("SPHINXINTL_TRANSLATOR", None)

# -- Options for HTML output
html_theme = "furo"
html_static_path = ["_static"]
html_title = f"{project} v{release}"
html_sidebars = {
    "**": [
        "sidebar/scroll-start.html",
        "sidebar/brand.html",
        "mini18n/snippets/select-lang.html",
        "sidebar/search.html",
        "sidebar/navigation.html",
        "sidebar/ethical-ads.html",
        "sidebar/scroll-end.html",
    ]
}

# -- Extension configuration
# For sphinx.ext.intersphinx
intersphinx_mapping = {
    "sphinx": ("https://www.sphinx-doc.org/en/master", None),
}
# atsphinx.mini18n
mini18n_default_language = "en"
mini18n_support_languages = ["en", "ja"]
mini18n_select_lang_label = ""
mini18n_basepath = "/audioplayer/"


def setup(app):
    app.add_object_type(
        "confval",
        "confval",
        objname="configuration value",
        indextemplate="pair: %s; configuration value",
    )



"""Configuration for pytest."""

from pathlib import Path

import pytest

pytest_plugins = "sphinx.testing.fixtures"


@pytest.fixture(scope="session")
def rootdir() -> Path:  # noqa: D103
    return Path(__file__).parent.resolve()



"""Register directive to render audio element."""

from docutils.parsers.rst import directives
from sphinx.application import Sphinx
from sphinx.directives import SphinxDirective
from sphinx.util.docutils import SphinxRole

from . import nodes

__version__ = "0.0.0"


class AudioDirective(SphinxDirective):  # noqa: D101
    required_arguments = 1
    option_spec = {
        "no-controls": directives.flag,
    }

    def run(self):  # noqa: D102
        attributes = {
            "uri": directives.uri(self.arguments[0]),
            "controls": self.options.get("no-controls", False),
        }
        node = nodes.audio(**attributes)
        return [
            node,
        ]


class AudioRole(SphinxRole):  # noqa: D101
    def run(self):  # noqa: D102
        options = {
            "uri": directives.uri(self.text),
            "controls": True,
        }
        node = nodes.audio(**options)
        return [node], []


def setup(app: Sphinx):  # noqa: D103
    app.add_node(nodes.audio, html=(nodes.visit_audio, nodes.depart_audio))
    app.add_directive("audio", AudioDirective)
    app.add_role("audio", AudioRole())
    return {
        "version": __version__,
        "env_version": 1,
        "parallel_read_safe": True,
        "parallel_write_safe": True,
    }



"""Standard tests."""

from io import StringIO

import pytest
from bs4 import BeautifulSoup
from sphinx.testing.util import SphinxTestApp


@pytest.mark.sphinx("html")
def test__directive(app: SphinxTestApp, status: StringIO, warning: StringIO):
    """Test to pass."""
    app.build()
    soup = BeautifulSoup((app.outdir / "index.html").read_text(), "html.parser")
    assert soup.audio
    assert soup.audio["src"] == "_images/dummy.mp3"
    assert soup.audio["controls"]


@pytest.mark.sphinx("html")
def test__domain(app: SphinxTestApp, status: StringIO, warning: StringIO):
    """Test to pass."""
    app.build()
    soup = BeautifulSoup((app.outdir / "use-domain.html").read_text(), "html.parser")
    assert soup.audio
    assert soup.audio["src"] == "_images/dummy.mp3"
    assert soup.audio["controls"]


@pytest.mark.sphinx("html")
def test__http_url(app: SphinxTestApp, status: StringIO, warning: StringIO):
    """Test to pass."""
    app.build()
    soup = BeautifulSoup((app.outdir / "http-url.html").read_text(), "html.parser")
    assert soup.audio
    assert (
        soup.audio["src"]
        == "https://github.com/atsphinx/audioplayer/raw/main/tests/test-root/dummy.mp3"
    )
    assert soup.audio["controls"]



# noqa: D100
from pathlib import Path
from urllib.parse import quote

from docutils import nodes
from sphinx.writers.html5 import HTML5Translator


class audio(nodes.image):  # noqa: D101
    pass


def visit_audio(self: HTML5Translator, node: audio):  # noqa: D103
    attributes = {}
    # Resolve audio file uri
    uri = node["uri"]
    if uri in self.builder.images:
        node["uri"] = Path(self.builder.imgpath) / quote(self.builder.images[uri])
    attributes["src"] = node["uri"]
    # Set boolean attributes
    if "controls" in node:
        attributes["controls"] = "controls"
    # Set starttag
    element = ""
    element += self.starttag(node, "audio", **attributes)
    self.body.append(element)


def depart_audio(self: HTML5Translator, node: audio):  # noqa: D103
    self.body.append("</audio>")
    pass
